@@ -4,8 +4,8 @@
 make dist
 
 # create tar archive and signature
-tarArchive=encfs-1.6-1.tgz
-mv encfs-1.6.tar.gz $tarArchive
+tarArchive=encfs-1.7.2.tgz
+mv encfs-1.7.2.tar.gz $tarArchive
 # let the user know why they're being asked for a passpharse
 echo "Signing tar archive - enter GPG password";
 gpg --detach-sign -a $tarArchive
@@ -19,8 +19,8 @@ gpg --detach-sign -a $tarArchive
 mkdir dist
 mv $tarArchive dist
 mv $tarArchive.asc dist
-#mv /usr/src/packages/SRPMS/encfs-1.6-1.src.rpm dist
-#mv /usr/src/packages/RPMS/i586/encfs-1.6-1.i586.rpm dist
+#mv /usr/src/packages/SRPMS/encfs-1.7.2-@RELEASE@.src.rpm dist
+#mv /usr/src/packages/RPMS/i586/encfs-1.7.2-@RELEASE@.i586.rpm dist
 
 # cleanup
 #rm /usr/src/packages/SOURCES/$tarArchive
